//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Draw_Library
//  Example1 - Default Device
//
// Date: 7th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to create an interface to the default directdraw
// device, how to retrieve it's capabilites, and finally,
// how to destroy the device and cleanup successfully
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDDCaps"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateDefaultButtonClick(TObject *Sender)
{
Msg("Creating Default Device");
Dx_Draw1->Create(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RetrieveCapsButtonClick(TObject *Sender)
{
Msg("Retrieving Device Capabilities");
if (Dx_Draw1->GetCaps( DriverCaps, HelCaps ))
    {
    // insert caps directly into listboxes
    HelCapsListBox->Items->Text = HelCaps->Caps_Strings->Text;
    DriverCapsListBox->Items->Text = DriverCaps->Caps_Strings->Text;

    // check for particular caps by comparing the 'public:' property using DirectX bitwise flags
    if (DriverCaps->Caps & DDCAPS_3D) Msg("Note: 3D Hardware Acceleration Available");
    if (DriverCaps->Caps & DDCAPS_BANKSWITCHED) Msg("Note: Display is Bank Switched Hardware: Relatively Slow Video Access");
    if (HelCaps->FXCaps & DDFXCAPS_BLTROTATION) Msg("Note: Hardware Emulation Layer supports Hardware Blit Rotation of arbitrary angle." );

    // [OR]
    // check for particular caps by comparing the '__published' property using the vcl's TSet<> operators
    if (DriverCaps->Caps_.Contains(ddcaps_bankswitched)) Msg("Note: Display is Bank Switched Hardware: Relatively Slow Video Access");
    if (DriverCaps->Caps_.Contains(ddcaps_blt)) Msg("Note: Hardware supports Bit-Blitting functions");
    if (HelCaps->FXCaps_.Contains(ddfxcaps_bltrotation90)) Msg("Note: Hardware Emulation Layer supports Hardware Blit Rotation of 90 degrees." );
    }
else
    {
    HelCapsListBox->Items->Text = "------ none ------";
    DriverCapsListBox->Items->Text = "------ none ------";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyDefaultButtonClick(TObject *Sender)
{
Msg("Destroying Default Device");
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
Msg("Device Created: 'Default DirectDraw Device'");
ActiveDeviceLabel->Caption = "-- Default --";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
Msg("Device Destroyed: 'Default DirectDraw Device'");
ActiveDeviceLabel->Caption = "-- none --";
HelCapsListBox->Items->Text = "------ none ------";
DriverCapsListBox->Items->Text = "------ none ------";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
AnsiString name;

if (Sender==Dx_Draw1) name = "Dx_Draw1"; else name = "unknown";

Msg("ERROR");
Msg(" An error occurred in the ["+name+"] component" );
Msg("" );
Msg("Error: "+Error );
Msg("Function: "+Function );
Msg("ErrorMessage: "+ErrorMessage );
}
//---------------------------------------------------------------------------







