//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Draw_Library, Example3 - Multiple Devices
//
// Date: 7th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example calls enumerates all devices using the TDx_Draw::DDEnumerate()
// method and the TDx_Draw::OnDDEnumerate() event.
//
// During each device enumeration it creates a TDx_Draw component using
// runtime code and stores the component into a global TList, a technique that
// can be utilized to create multiple directdraw devices.
//
// After enumeration it displays the capabilities of each device before finally
// destroying all devices.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\ExtCtrls.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDCaps.h"
#include "TDDPixelFormat.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 +
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TPanel *Panel1;
    TPanel *Panel2;
    TPanel *Panel3;
    TPanel *Panel4;
    TPanel *Panel5;

    TLabel *Label2;
    TLabel *Label1;
    TLabel *Label4;
    TLabel *Label3;
    TLabel *Label5;
    TLabel *Label8;

    TMemo *InfoMemo;
    TMemo *StatusMemo;

    TListBox *HelCapsListBox;
    TListBox *DevicesListBox;
    TListBox *DriverCapsListBox;

    TBitBtn *EnumerateButton;
    TBitBtn *SelectAllDevicesButton;
    TBitBtn *CreateSelectedButton;
    TBitBtn *RetrieveCapsButton;
    TBitBtn *DestroyActiveButton;
    TBitBtn *ClearDevicesButton;
    TBitBtn *ExitButton;
    TBitBtn *ClearButton;

    TDDCaps *HelCaps;
    TDDCaps *DriverCaps;
    TDx_Draw *Dx_Draw1;

    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall EnumerateButtonClick(TObject *Sender);
    void __fastcall SelectAllDevicesButtonClick(TObject *Sender);
    void __fastcall CreateSelectedButtonClick(TObject *Sender);
    void __fastcall DevicesListBoxClick(TObject *Sender);
    void __fastcall RetrieveCapsButtonClick(TObject *Sender);
    void __fastcall DestroyActiveButtonClick(TObject *Sender);
    void __fastcall ClearDevicesButtonClick(TObject *Sender);

    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearButtonClick(TObject *Sender);

    void __fastcall Dx_Draw1Create(TObject *Sender);
    void __fastcall Dx_Draw1Destroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);

    void __fastcall Dx_Draw1DDEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverName, void *Context,
    HMONITOR HMonitor, bool &Finished);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

    #define MAX_DEVICES 5

    AnsiString  Names[MAX_DEVICES];
    AnsiString  Descriptions[MAX_DEVICES];
    GUID*       Guids[MAX_DEVICES];
    int         NumDevices;

    TList*      TDx_Draw_Component_List;
};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif

