//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Draw_Library, Example5 - Display Modes
//
// Date: 7th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Desciption:
//
// This example shows how to Enumerate all available display modes for the
// default device using the TDx_Draw::EnumDisplayModes() method and the
// TDx_Draw::OnEnumDisplayModes() event.
//
// Once enumerated it allows you to set any of the available display modes
// using the TDx_Draw::SetDisplayMode() method.
//
// Finally, it resets the original display mode (at application startup)
// using TDx_Draw::RestoreDisplayMode().
//
// NOTE: It also contains code to determine the 'bits_per_pixel' of the display
// mode based on the mode's surface description.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\ExtCtrls.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDCaps.h"
#include "TDDPixelFormat.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 +
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TPanel *Panel1;
    TPanel *Panel2;
    TPanel *Panel3;
    TPanel *Panel4;
    TPanel *Panel5;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label5;

    TMemo *InfoMemo;
    TMemo *StatusMemo;

    TBitBtn *CreateDefaultButton;
    TBitBtn *EnumModesButton;
    TBitBtn *DestroyDefaultButton;
    TBitBtn *ExitButton;
    TBitBtn *ClearButton;
    TDx_Draw *Dx_Draw1;
    TBitBtn *NextModeButton;
    TBitBtn *SetModeButton;
    TBitBtn *RestoreModeButton;
    TListBox *ModesListBox;
    TLabel *Label7;
    TLabel *CurrentModeLabel;
    TLabel *Label1;

    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall CreateDefaultButtonClick(TObject *Sender);
    void __fastcall RestoreModeButtonClick(TObject *Sender);
    void __fastcall DestroyDefaultButtonClick(TObject *Sender);

    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearButtonClick(TObject *Sender);

    void __fastcall Dx_Draw1Create(TObject *Sender);
    void __fastcall Dx_Draw1Destroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall Dx_Draw1EnumDisplayModes(TObject *Sender,
          TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished);
    void __fastcall EnumModesButtonClick(TObject *Sender);
    void __fastcall SetModeButtonClick(TObject *Sender);
    void __fastcall NextModeButtonClick(TObject *Sender);
    void __fastcall ModesListBoxClick(TObject *Sender);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

    #define MAX_MODES 50

    int         Widths[MAX_MODES];
    int         Heights[MAX_MODES];
    int         BitDepths[MAX_MODES];
    int         RefreshRates[MAX_MODES];
    int         NumModes;

};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif

