//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Draw_Library,
//       ExampleB - Loading and Saving Surfaces
//
// Date: 7th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to use the TDx_DrawSurface::CreateFromFile()
// and TDx_DrawSurface::SaveToFile() methods, and also shows how to
// create many offscreen surfaces that can be pre-loaded and painted
// wherever to create the end-result display.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_DrawPalette"
#pragma link "tddbltfx"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
AnsiString IntToHexStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
AnsiString string = "0x";
int value = 0x10000000;
for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Top=0;
Left=0;
Width=800;
Height=600;
CurrentPageNumber = 0;

canvas = new TCanvas();
if (canvas==NULL) ShowMessage("Could not create VCL canvas");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
if (canvas!=NULL) delete canvas;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
ModesListBox->Items->Clear();
NumModes=0;
Dx_Draw1->EnumDisplayModes(0,NULL,NULL);
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE );

// create: 1 x Complex Primary Surface, with 1 x attached Back Buffer, in an auto-flip configuration
PrimaryCaps->Caps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
PrimaryDesc->SCaps = PrimaryCaps;
PrimaryDesc->Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
PrimaryDesc->BackBufferCount = 1;
PrimarySurface->Create( PrimaryDesc, Dx_Draw1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
PrimarySurface->Destroy();
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_NORMAL );
if (Dx_Draw1->RestoreDisplayMode())
    {
    Top=0;
    Left=0;
    Width=800;
    Height=600;
    Invalidate();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1EnumDisplayModes(TObject *Sender,
      TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished)
{
int width = SurfaceDesc->Width;
int height = SurfaceDesc->Height;
int refresh_rate = SurfaceDesc->RefreshRate;
int bit_depth;

if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = SurfaceDesc->PixelFormat->RGBBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = SurfaceDesc->PixelFormat->YUVBitCount[0];

if (NumModes<MAX_MODES)
    {
    Widths[NumModes] = width;
    Heights[NumModes] = height;
    BitDepths[NumModes] = bit_depth;
    RefreshRates[NumModes] = refresh_rate;
    NumModes++;

    ModesListBox->Items->Add( IntToStr(width)+"x"+IntToStr(height)+"x"+IntToStr(bit_depth)+" @ "+IntToStr(refresh_rate)+"Hz" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::ModesListBoxClick(TObject *Sender)
{
if (ModesListBox->ItemIndex!=-1)
    {
    int index = ModesListBox->ItemIndex;
    AnsiString name = ModesListBox->Items->Strings[index];

    Msg("Mode Selected: '"+name+"'");
    CurrentModeLabel->Caption = name;
    }
else
    CurrentModeLabel->Caption = "-- default --";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceCreate(TObject *Sender)
{
Msg(">>> Primary Surface Created");

// get it's attached backbuffer
if (PrimarySurface->GetAttachedSurface( BackCaps, BackBuffer ))
    // call the backbuffer::OnCreate() event manually.
    BackBuffer->OnCreate(this);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceDestroy(TObject *Sender)
{
Msg(">>> Primary Surface Destroy");
if (BackBuffer->Created) BackBuffer->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceSurfaceLost(TObject *Sender)
{
Dx_Draw1->RestoreAllSurfaces();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ModesListBoxDblClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    ShowMessage("Please Note:\n\
You may experience difficulty restoring your display mode.\n\
To return to the original display mode at any time, press ALT-R (Restore) or ALT-X (eXit)" );

    PrimarySurface->Destroy();

    int index = ModesListBox->ItemIndex;
    Dx_Draw1->SetDisplayMode( Widths[index], Heights[index], BitDepths[index], RefreshRates[index], 0 );

    PrimarySurface->Create( PrimaryDesc, Dx_Draw1 );
    RepaintButtonClick(this);
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::RepaintButtonClick(TObject *Sender)
{
Application->OnIdle = NULL;
if (Dx_Draw1->Created) Dx_Draw1->FlipToGDISurface();

    int old_top = Top;
    Top=Screen->Height+1;
    Invalidate();
    Top=old_top;
    Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferCreate(TObject *Sender)
{
Msg(">>> BackBuffer Created");
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferDestroy(TObject *Sender)
{
Msg(">>> BackBuffer Destroyed");
for (int i=0;i<OpenDialog1->Files->Count;i++)
    {
    if (OffscreenSurfaces[i])
        {
        if (OffscreenSurfaces[i]->Created)
            OffscreenSurfaces[i]->Destroy();
        delete OffscreenSurfaces[i];
        OffscreenSurfaces[i] = NULL;
        }
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::BitBtn2Click(TObject *Sender)
{
Dx_Draw1->Create(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PageFlipButtonClick(TObject *Sender)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

ShowMessage("Please Note:\n\
After the primary surface has been painted the contents of the current form\n\
will no longer be visible though it is still active. To repaint the form\n\
\n\
Press ALT-S (Snapshot) to save a snapshot of the screen to the image source directory\n\
Press ALT-R (Restore) or to exit the application, press ALT-X (eXit)" );

randomize();
CurrentPageNumber = 0;
Application->OnIdle = OnIdle;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnIdle(TObject *Sender, bool& Done)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// this method contains the actual paint code.
// it :-
// (1) paints the entire backbuffer by stretching the pre-rendered offscreen surface so that it fills the entire backbuffer
// (2) paints some text over the top of the newly painted backbuffer
// (3) flips the backbuffer to the primary surface, displaying the newly painted backbuffer.


// paint the entire backbuffer BLACK
DDBltFX1->FillColor = ColorToRGB( clBlack );
BackBuffer->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, DDBltFX1 );

// tile the images onto the backbuffer
TRect r;
for (int i=0;i<OpenDialog1->Files->Count;i++)
    {
    r.Top = random(Screen->Height-100);
    r.Left = random(Screen->Width-100);
    r.Right = r.Left + 100;
    r.Bottom = r.Top + 100;
    if (OffscreenSurfaces[i])
        BackBuffer->Blt( &r, OffscreenSurfaces[i], NULL, 0, NULL );
    }

// paint some text over the top
HDC hdc;
if (BackBuffer->GetDC(&hdc))
    {
    try
        {
        // make sure to reassign the canvas so it paints in the right spot
        canvas->Handle = hdc;

        // paint some text
        canvas->Brush->Style = bsClear;
        canvas->Pen->Style = psClear;
        canvas->Font->Name = "Arial";
        canvas->Font->Style = TFontStyles();
        canvas->Font->Size = 20;
        canvas->Font->Color = (TColor) RGB(0,0,0);
        canvas->TextOut( 2, 2, "Paint the BackBuffer with heaps of OffscreenSurfaces, then Flip" );
        canvas->Font->Color = (TColor) RGB(128,255,128);
        canvas->TextOut( 0, 0, "Paint the BackBuffer with heaps of OffscreenSurfaces, then Flip" );

        // and more text
        canvas->Brush->Style = bsClear;
        canvas->Font->Name = "Comic Sans MS";
        canvas->Font->Style = TFontStyles() << fsBold << fsItalic << fsUnderline << fsStrikeOut;
        canvas->Font->Size = 24;
        canvas->Font->Color = (TColor) RGB(0,0,0);
        canvas->TextOut( 102 + CurrentPageNumber, 42, "Frame# "+IntToStr(CurrentPageNumber) );
        canvas->Font->Color = (TColor) RGB(128+CurrentPageNumber%128,255,128+CurrentPageNumber%128);
        canvas->TextOut( 100 + CurrentPageNumber, 40, "Frame# "+IntToStr(CurrentPageNumber) );

        // and remove our temporary canvas handle
        canvas->Handle = NULL;
        }
    catch (Exception &e)
        {
        Application->OnIdle = NULL;
        }

    BackBuffer->ReleaseDC(hdc);
    }

// display the result
PrimarySurface->Flip( BackBuffer, DDFLIP_WAIT );

// increment our animation counter
CurrentPageNumber++;

// continue animating asap
Done = false;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ChooseButtonClick(TObject *Sender)
{
if (OpenDialog1->Execute())
    {
    SelectedLabel->Caption = "Selected:  0 images";
    SelectedLabel->Update();
    int created_count = 0;

    for (int i=0;i<OpenDialog1->Files->Count;i++)
        {
        if (OffscreenSurfaces[i])
            {
            if (OffscreenSurfaces[i]->Created)
                OffscreenSurfaces[i]->Destroy();
            delete OffscreenSurfaces[i];
            OffscreenSurfaces[i] = NULL;
            }

        OffscreenSurfaces[i] = new TDx_DrawSurface( this );
        if (OffscreenSurfaces[i])
            try {
                OffscreenSurfaces[i]->OnError = Dx_Draw1->OnError;
                OffscreenSurfaces[i]->OnSurfaceLost = OnSurfaceLost;
                Msg( "Creating Surface #"+IntToStr(i)+" from file: "+OpenDialog1->Files->Strings[i]);
                if (OffscreenSurfaces[i]->CreateFromFile( Dx_Draw1, OpenDialog1->Files->Strings[i] ))
                    {
                    CreatedLabel->Caption = "Created:    "+IntToStr(++created_count)+" offscreen surfaces";
                    CreatedLabel->Update();
                    }
                }
            catch (...)
                {
                Msg( "Error:: COULD NOT CREATE FILE # "+IntToStr(i) );
                delete OffscreenSurfaces[i];
                OffscreenSurfaces[i] = NULL;
                }
        }
    }
}
// --------------------------------------------------------------------------

void __fastcall TMainForm::OnSurfaceLost(TObject *Sender)
{
for (int i=0;i<OpenDialog1->Files->Count;i++)
    {
    if (OffscreenSurfaces[i])
        {
        OffscreenSurfaces[i]->Restore();

        if (OffscreenSurfaces[i]->Created)
            OffscreenSurfaces[i]->Destroy();
        
        OffscreenSurfaces[i]->CreateFromFile( Dx_Draw1, OpenDialog1->Files->Strings[i] );
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveButtonClick(TObject *Sender)
{
for (int i=0;i<OpenDialog1->Files->Count;i++)
    {
    if (OffscreenSurfaces[i])
        {
        try {
            Msg( "Saving Surface #"+IntToStr(i)+" to file: "+OpenDialog1->Files->Strings[i]+".TDX.BMP");
            OffscreenSurfaces[i]->SaveToFile( Dx_Draw1, OpenDialog1->Files->Strings[i]+".TDX.BMP" );
            }
        catch (...)
            {
            Msg( "Error:: COULD NOT SAVE FILE # "+IntToStr(i) );
            }
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ReloadButtonClick(TObject *Sender)
{
for (int i=0;i<OpenDialog1->Files->Count;i++)
    {
    if (OffscreenSurfaces[i])
        {
        if (OffscreenSurfaces[i]->Created)
            OffscreenSurfaces[i]->Destroy();

        try {
            Msg( "Creating Surface #"+IntToStr(i)+" from file: "+OpenDialog1->Files->Strings[i]+".TDX.BMP");
            OffscreenSurfaces[i]->CreateFromFile( Dx_Draw1, OpenDialog1->Files->Strings[i]+".TDX.BMP" );
            }
        catch (...)
            {
            Msg( "Error:: COULD NOT CREATE FILE # "+IntToStr(i) );
            }
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SnapshotButtonClick(TObject *Sender)
{
try {
    if (PrimarySurface->Created)                  
        if (PrimarySurface->SaveToFile( Dx_Draw1, GetCurrentDir() + "\\ExampleB_Snapshot.BMP" ))
            Msg("Saved screen snapshot to file: "+GetCurrentDir() + "\\ExampleB_Snapshot.BMP" );
    }
catch (...)
    {
    Msg("Error: Could not save a snapshot to file: " + GetCurrentDir() + "\\ExampleB_Snapshot.BMP" )
    RepaintButtonClick(this);
    }
}
//---------------------------------------------------------------------------
