//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Draw_Library,
//       ExampleB - Loading and Saving Surfaces
//
// Date: 7th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to use the TDx_DrawSurface::CreateFromFile()
// and TDx_DrawSurface::SaveToFile() methods, and also shows how to
// create many offscreen surfaces that can be pre-loaded and painted
// wherever to create the end-result display.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\ExtCtrls.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDCaps.h"
#include "TDDPixelFormat.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
#include <Dialogs.hpp>
#include <Graphics.hpp>
#include "TDx_DrawPalette.h"
#include "tddbltfx.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 +
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TPanel *Panel1;
    TPanel *Panel2;
    TPanel *Panel3;
    TPanel *Panel4;
    TPanel *Panel5;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label5;

    TMemo *InfoMemo;
    TMemo *StatusMemo;
    TBitBtn *ExitButton;
    TBitBtn *ClearButton;
    TDx_Draw *Dx_Draw1;
    TListBox *ModesListBox;
    TLabel *Label7;
    TLabel *CurrentModeLabel;
    TLabel *Label1;
    TDx_DrawSurface *PrimarySurface;
    TButton *PageFlipButton;
    TButton *RepaintButton;
    TBitBtn *BitBtn2;
    TDx_DrawSurface *BackBuffer;
    TBitBtn *ChooseButton;
    TLabel *SelectedLabel;
    TOpenDialog *OpenDialog1;
    TLabel *Label4;
    TBitBtn *SaveButton;
    TDDBltFX *DDBltFX1;
    TBitBtn *ReloadButton;
    TBitBtn *SnapshotButton;
    TLabel *CreatedLabel;
    TDDSurfaceDesc *PrimaryDesc;
    TDDSCaps *PrimaryCaps;
    TDDSCaps *BackCaps;
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearButtonClick(TObject *Sender);

    void __fastcall Dx_Draw1Create(TObject *Sender);
    void __fastcall Dx_Draw1Destroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall Dx_Draw1EnumDisplayModes(TObject *Sender,
          TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished);
    
    void __fastcall ModesListBoxClick(TObject *Sender);
    void __fastcall PrimarySurfaceCreate(TObject *Sender);
    void __fastcall PrimarySurfaceDestroy(TObject *Sender);
    void __fastcall PrimarySurfaceSurfaceLost(TObject *Sender);
    void __fastcall ModesListBoxDblClick(TObject *Sender);
    
    void __fastcall RepaintButtonClick(TObject *Sender);
    void __fastcall BackBufferCreate(TObject *Sender);
    void __fastcall BackBufferDestroy(TObject *Sender);
    void __fastcall BitBtn2Click(TObject* Sender);
    void __fastcall PageFlipButtonClick(TObject *Sender);
    void __fastcall ChooseButtonClick(TObject *Sender);
    void __fastcall SaveButtonClick(TObject *Sender);
    void __fastcall SnapshotButtonClick(TObject *Sender);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
    void __fastcall ReloadButtonClick(TObject *Sender);
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

    void __fastcall OnIdle(TObject *Sender, bool& Done);
    void __fastcall OnSurfaceLost(TObject *Sender);

    #define MAX_MODES 50

    int         Widths[MAX_MODES];
    int         Heights[MAX_MODES];
    int         BitDepths[MAX_MODES];
    int         RefreshRates[MAX_MODES];
    int         NumModes;

    int         CurrentPageNumber;
    TCanvas*    canvas;

    #define MAX_IMAGES    5000

//    TList*      Images;
//    TList*      Images_Widths;
//    TList*      Images_Heights;
//    TList*      Images_Bits;

    TDx_DrawSurface*    OffscreenSurfaces[MAX_IMAGES];

};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif

